@@ -3,7 +3,7 @@
 import sys, os
 basedir = os.path.abspath(os.path.dirname(__file__))
 
-from crcgen.version import VERSION_STRING
+from libcrcgen.version import VERSION_STRING
 from distutils.core import setup
 import warnings
 
@@ -20,12 +20,12 @@ setup(	name		= "crcgen",
 	author_email	= "m@bues.ch",
 	url		= "https://bues.ch/h/crcgen",
 	scripts		= [
-		"scripts/crcgen",
-	],
-	packages	= [
 		"crcgen",
 	],
-	keywords	= "CRC Verilog FPGA codegenerator",
+	packages	= [
+		"libcrcgen",
+	],
+	keywords	= "CRC Verilog VHDL MyHDL FPGA codegenerator",
 	classifiers	= [
 		"Development Status :: 5 - Production/Stable",
 		"Environment :: Console",
@@ -35,6 +35,7 @@ setup(	name		= "crcgen",
 		"Intended Audience :: Science/Research",
 		"Intended Audience :: Telecommunications Industry",
 		"License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
+		"License :: Public Domain",
 		"Operating System :: OS Independent",
 		"Programming Language :: Python",
 		"Programming Language :: Python :: 3",
