@@ -2,7 +2,7 @@
 #
 # PROFIBUS DP - Master
 #
-# Copyright (c) 2013-2020 Michael Buesch <m@bues.ch>
+# Copyright (c) 2013-2021 Michael Buesch <m@bues.ch>
 #
 # Licensed under the terms of the GNU General Public License version 2,
 # or (at your option) any later version.
@@ -37,12 +37,12 @@ class DpSlaveState(object):
 	STATE_DX	= 5 # Data_Exchange
 
 	state2name = {
-		_STATE_INVALID	: "invalid",
-		STATE_INIT	: "init",
-		STATE_WDIAG	: "wait for diag",
-		STATE_WPRM	: "wait for Prm",
-		STATE_WCFG	: "wait for Cfg",
-		STATE_WDXRDY	: "wait for Data_Exchange-ready",
+		_STATE_INVALID	: "Invalid",
+		STATE_INIT	: "Init",
+		STATE_WDIAG	: "Wait for diag",
+		STATE_WPRM	: "Wait for Prm",
+		STATE_WCFG	: "Wait for Cfg",
+		STATE_WDXRDY	: "Request diag and wait for DX-ready",
 		STATE_DX	: "Data_Exchange",
 	}
 
@@ -62,11 +62,13 @@ class DpSlaveState(object):
 		"__state",
 		"__stateTimeout",
 		"dxStartTime",
+		"dxCount",
+		"dxCycleRunning",
 		"faultDeb",
 		"fcb",
-		"inData",
 		"master",
-		"outData",
+		"fromSlaveData",
+		"toSlaveData",
 		"pendingReq",
 		"pendingReqTimeout",
 		"rxQueue",
@@ -98,13 +100,14 @@ class DpSlaveState(object):
 
 		# Data_Exchange context
 		self.dxStartTime = 0.0
+		self.dxCount = 0
 
 		# Received telegrams
 		self.rxQueue = []
 
 		# In/Out user data
-		self.outData = None
-		self.inData = None
+		self.toSlaveData = None
+		self.fromSlaveData = None
 
 	def getRxQueue(self):
 		rxQueue = self.rxQueue
@@ -123,6 +126,8 @@ class DpSlaveState(object):
 	def setState(self, state, stateTimeLimit=None):
 		if stateTimeLimit is None:
 			stateTimeLimit = self.stateTimeLimits[state]
+		if state == self.STATE_INIT:
+			self.dxCycleRunning = False
 		self.__nextState = state
 		self.__stateTimeout.start(stateTimeLimit)
 		self.master.phy.clearTxQueueAddr(self.slaveDesc.slaveAddr)
@@ -156,42 +161,44 @@ class DpSlaveDesc(object):
 	"""
 
 	__slots__ = (
-		"index",
-		"name",
-		"chkCfgTelegram",
 		"dpm",
 		"gsd",
-		"identNumber",
-		"setPrmTelegram",
-		"setPrmTelegram",
 		"slaveAddr",
+		"identNumber",
+		"name",
+		"index",
+		"inputSize",
+		"outputSize",
+		"diagPeriod",
 		"slaveConf",
 		"userData",
+		"setPrmTelegram",
+		"chkCfgTelegram",
 	)
 
-	def __init__(self,
-		     gsd,
-		     slaveAddr,
-		     slaveConf=None):
-		self.gsd = gsd
-		self.slaveAddr = slaveAddr
-		self.slaveConf = slaveConf
-		self.identNumber = gsd.getIdentNumber() if gsd else 0
+	def __init__(self, slaveConf=None):
 		self.dpm = None
-		self.name = None
-		self.index = None
+		self.gsd = slaveConf.gsd if slaveConf else None
+		self.slaveAddr = slaveConf.addr if slaveConf else None
+		self.identNumber = self.gsd.getIdentNumber() if self.gsd else 0
+		self.name = slaveConf.name if slaveConf else None
+		self.index = slaveConf.index if slaveConf else None
+		self.inputSize = slaveConf.inputSize if slaveConf else 0
+		self.outputSize = slaveConf.outputSize if slaveConf else 0
+		self.diagPeriod = slaveConf.diagPeriod if slaveConf else 0
+		self.slaveConf = slaveConf
 		self.userData = {} # For use by application code.
 
 		# Prepare a Set_Prm telegram.
 		self.setPrmTelegram = DpTelegram_SetPrm_Req(
-					da = self.slaveAddr,
-					sa = None)
+					da=self.slaveAddr,
+					sa=None)
 		self.setPrmTelegram.identNumber = self.identNumber
 
 		# Prepare a Chk_Cfg telegram.
 		self.chkCfgTelegram = DpTelegram_ChkCfg_Req(
-					da = self.slaveAddr,
-					sa = None)
+					da=self.slaveAddr,
+					sa=None)
 
 	def setCfgDataElements(self, cfgDataElements):
 		"""Sets DpCfgDataElement()s from the specified list
@@ -257,11 +264,39 @@ class DpSlaveDesc(object):
 		self.setPrmTelegram.wdFact1 = fact1
 		self.setPrmTelegram.wdFact2 = fact2
 
+	def setMasterOutData(self, data):
+		"""Set the master-out-data that will be sent the
+		next time we are able to send something to that slave.
+		"""
+		self.dpm._setToSlaveData(self, data)
+
+	def getMasterInData(self):
+		"""Get the latest received master-in-data.
+		Returns None, if there was no received data.
+		"""
+		return self.dpm._getFromSlaveData(self)
+
 	def setOutData(self, outData):
-		self.dpm.setSlaveOutData(self, outData)
+		"""Deprecated: Don't use this method. Use setMasterOutData() instead."""
+		self.setMasterOutData(outData)
 
 	def getInData(self):
-		return self.dpm.getSlaveInData(self)
+		"""Deprecated: Don't use this method. Use getMasterInData() instead."""
+		return self.getMasterInData()
+
+	def isConnecting(self):
+		"""Returns True, if the slave is in the process of getting connected/configured,
+		but is not fully connected, yet.
+		Otherwise returns False.
+		"""
+		return self.dpm._slaveIsConnecting(self)
+
+	def isConnected(self):
+		"""Returns True, if this slave is fully connected and Data_Exchange
+		or periodic slave diagnosis is currently running.
+		Otherwise returns False.
+		"""
+		return self.dpm._slaveIsConnected(self)
 
 	def __repr__(self):
 		return "DpSlaveDesc(identNumber=%s, slaveAddr=%d)" %\
@@ -300,8 +335,8 @@ class DpMaster(object):
 		self.fdlTrans = FdlTransceiver(self.phy)
 		self.dpTrans = DpTransceiver(self.fdlTrans, thisIsMaster=True)
 
-		mcastSlaveDesc = DpSlaveDesc(gsd=None,
-					     slaveAddr=FdlTelegram.ADDRESS_MCAST)
+		mcastSlaveDesc = DpSlaveDesc()
+		mcastSlaveDesc.slaveAddr = FdlTelegram.ADDRESS_MCAST
 		mcastSlave = DpSlaveState(self, mcastSlaveDesc)
 
 		self.__slaveDescs = {
@@ -344,6 +379,10 @@ class DpMaster(object):
 	def addSlave(self, slaveDesc):
 		"""Register a slave."""
 
+		if slaveDesc.inputSize <= 0:
+			raise DpError("Slave %d: input_size=0 is currently not supported." % (
+				      slaveDesc.slaveAddr))
+
 		slaveAddr = slaveDesc.slaveAddr
 		if slaveAddr in self.__slaveDescs or\
 		   slaveAddr in self.__slaveStates:
@@ -521,42 +560,54 @@ class DpMaster(object):
 						self.__errorMsg("Slave %d is not reachable "
 							"via this line." %\
 							slave.slaveDesc.slaveAddr)
+						slave.faultDeb.fault()
 					if telegram.cfgFault():
 						self.__errorMsg("Slave %d reports a faulty "
 							"configuration (Chk_Cfg)." %\
 							slave.slaveDesc.slaveAddr)
+						slave.faultDeb.fault()
 					if telegram.prmFault():
 						self.__errorMsg("Slave %d reports a faulty "
 							"parameterization (Set_Prm)." %\
 							slave.slaveDesc.slaveAddr)
+						slave.faultDeb.fault()
+					if telegram.prmReq():
+						self.__debugMsg("Slave %d requests a new "
+							"parameterization (Set_Prm)." %\
+							slave.slaveDesc.slaveAddr)
+						slave.faultDeb.fault()
 					if telegram.isNotSupp():
 						self.__errorMsg("Slave %d replied with "
 							"\"function not supported\". "
 							"The parameters should be checked "
 							"(Set_Prm)." %\
 							slave.slaveDesc.slaveAddr)
+						slave.faultDeb.fault()
 					if telegram.masterLock():
 						self.__errorMsg("Slave %d is already controlled "
 							"(locked to) another DP-master." %\
 							slave.slaveDesc.slaveAddr)
+						slave.faultDeb.fault()
 					if not telegram.hasOnebit():
 						self.__debugMsg("Slave %d diagnostic "
 							"always-one-bit is zero." %\
 							slave.slaveDesc.slaveAddr)
-
+						slave.faultDeb.fault()
 					if telegram.hasExtDiag():
 						pass#TODO turn on red DIAG-LED
+						slave.faultDeb.fault()
+
 					if telegram.isReadyDataEx():
 						slave.setState(slave.STATE_DX)
 						return None
-					elif telegram.needsNewPrmCfg():
+					if telegram.needsNewPrmCfg():
 						slave.setState(slave.STATE_INIT)
 						return None
 					break
 				else:
 					self.__debugMsg("Received spurious "
 						"telegram:\n%s" % str(telegram))
-
+					slave.faultDeb.fault()
 		if (not slave.pendingReq or
 		    slave.pendingReqTimeout.exceed()):
 			ok = self.__send(slave,
@@ -566,79 +617,113 @@ class DpMaster(object):
 					 timeout=0.05)
 			if not ok:
 				self.__debugMsg("SlaveDiag_Req failed")
+				slave.faultDeb.fault()
 				return None
+		self.__checkFaultDeb(slave, False)
 		return None
 
 	def __runSlave_dataExchange(self, slave):
-		#TODO: add support for in/out-only slaves
 		dataExInData = None
 
 		if slave.stateJustEntered():
-			self.__debugMsg("Initialization finished. "
-				"Running Data_Exchange with slave %d..." %\
-				slave.slaveDesc.slaveAddr)
+			self.__debugMsg("%sRunning Data_Exchange with slave %d..." % (
+				"" if slave.dxCycleRunning else "Initialization finished. ",
+				slave.slaveDesc.slaveAddr))
 			slave.flushRxQueue()
+			slave.faultDeb.ok()
 			slave.dxStartTime = monotonic_time()
+			slave.dxCycleRunning = True
+			slave.dxCount = 0
 
+		slaveOutputSize = slave.slaveDesc.outputSize
 		if slave.pendingReq:
 			for telegram in slave.getRxQueue():
-				if not DpTelegram_DataExchange_Con.checkType(telegram):
+				if slaveOutputSize == 0:
+					# This slave should not send any data.
 					self.__debugMsg("Ignoring telegram in "
 						"DataExchange with slave %d:\n%s" %(
 						slave.slaveDesc.slaveAddr, str(telegram)))
 					slave.faultDeb.fault()
 					continue
-				resFunc = telegram.fc & FdlTelegram.FC_RESFUNC_MASK
-				if resFunc in (FdlTelegram.FC_DH,
-					       FdlTelegram.FC_RDH,):
-					self.__debugMsg("Slave %d requested diagnostics." %\
-						slave.slaveDesc.slaveAddr)
-					slave.setState(slave.STATE_WDXRDY, 0.2)
-				elif resFunc == FdlTelegram.FC_RS:
-					raise DpError("Service not active "
-						"on slave %d" % slave.slaveDesc.slaveAddr)
-				dataExInData = telegram.getDU()
-
-			if dataExInData is None:
+				else:
+					# This slave is supposed to send some data.
+					# Get it.
+					if not DpTelegram_DataExchange_Con.checkType(telegram):
+						self.__debugMsg("Ignoring telegram in "
+							"DataExchange with slave %d:\n%s" %(
+							slave.slaveDesc.slaveAddr, str(telegram)))
+						slave.faultDeb.fault()
+						continue
+					resFunc = telegram.fc & FdlTelegram.FC_RESFUNC_MASK
+					if resFunc in (FdlTelegram.FC_DH, FdlTelegram.FC_RDH):
+						self.__debugMsg("Slave %d requested diagnostics." %\
+							slave.slaveDesc.slaveAddr)
+						slave.setState(slave.STATE_WDXRDY, 0.2)
+					elif resFunc == FdlTelegram.FC_RS:
+						raise DpError("Service not active "
+							"on slave %d" % slave.slaveDesc.slaveAddr)
+					dataExInData = telegram.getDU()
+			if (dataExInData is not None or
+			    (slaveOutputSize == 0 and slave.shortAckReceived)):
+				# We received some data or an ACK (input-only slave).
+				slave.pendingReq = None
+				slave.faultDeb.ok()
+				slave.restartStateTimeout()
+				self._releaseSlave(slave)
+			else:
+				# No data or ACK received from slave.
 				if slave.pendingReqTimeout.exceed():
 					self.__debugMsg("Data_Exchange timeout with slave %d" % (
 							slave.slaveDesc.slaveAddr))
 					slave.faultDeb.fault()
 					slave.pendingReq = None
-			else:
-				# We received some data.
-				slave.pendingReq = None
-				slave.faultDeb.ok()
-				slave.restartStateTimeout()
-				self._releaseSlave(slave)
 		else:
-			# Send the out data telegram, if any.
-			outData = slave.outData
-			if outData is not None:
-				ok = self.__send(slave,
-						 telegram=DpTelegram_DataExchange_Req(
-							da=slave.slaveDesc.slaveAddr,
-							sa=self.masterAddr,
-							du=outData),
-						 timeout=0.1)
-				if not ok:
-					self.__debugMsg("DataExchange_Req failed")
-					return None
-				# We sent it. Reset the data.
-				slave.outData = None
+			diagPeriod = slave.slaveDesc.diagPeriod
+			if diagPeriod > 0 and slave.dxCount >= diagPeriod:
+				# The input-only slave shall periodically be diagnosed.
+				# Go to diagnostic state.
+				slave.setState(slave.STATE_WDXRDY, 0.2)
+			else:
+				# Send the out data telegram, if any.
+				toSlaveData = slave.toSlaveData
+				if toSlaveData is not None:
+					if slave.slaveDesc.inputSize == 0:
+						self.__debugMsg("Got data for slave, "
+								"but slave does not expect any input data.")
+					else:
+						ok = self.__send(slave,
+								 telegram=DpTelegram_DataExchange_Req(
+									da=slave.slaveDesc.slaveAddr,
+									sa=self.masterAddr,
+									du=toSlaveData),
+								 timeout=0.1)
+						if ok:
+							# We sent it. Reset the data.
+							slave.toSlaveData = None
+							slave.dxCount = min(slave.dxCount + 1, 0x3FFFFFFF)
+						else:
+							self.__debugMsg("DataExchange_Req failed")
+							slave.faultDeb.fault()
+		if self.__checkFaultDeb(slave, True):
+			return None
+		return dataExInData
 
+	def __checkFaultDeb(self, slave, inDataExchange):
 		faultCount = slave.faultDeb.get()
 		if faultCount >= 5:
 			# communication lost
-			self.__debugMsg("Communication lost in Data_Exchange.")
+			self.__debugMsg("Communication lost in Data_Exchange or Slave_Diag.")
 			slave.setState(slave.STATE_INIT)
-		elif faultCount >= 3 and monotonic_time() >= slave.dxStartTime + 0.2:
+			return True
+		elif (faultCount >= 3 and
+		      inDataExchange and
+		      (monotonic_time() >= slave.dxStartTime + 0.2 or slave.slaveDesc.outputSize == 0)):
 			# Diagnose the slave
 			self.__debugMsg("Many errors in Data_Exchange. "
-				"Requesting diagnostic information...")
+					"Requesting diagnostic information...")
 			slave.setState(slave.STATE_WDXRDY, 0.2)
-
-		return dataExInData
+			return True
+		return False
 
 	__slaveStateHandlers = {
 		DpSlaveState.STATE_INIT		: __runSlave_init,
@@ -739,25 +824,51 @@ class DpMaster(object):
 		self.__runNextSlaveIndex = (runNextSlaveIndex + 1) % len(slaveDescsList)
 
 		slave = self.__slaveStates[slaveDesc.slaveAddr]
-		slave.inData = self.__runSlave(slave)
+		fromSlaveData = self.__runSlave(slave)
+		if (fromSlaveData is not None and
+		    len(fromSlaveData) != slaveDesc.outputSize):
+			self.__errorMsg("Slave %d: The received data size (%d bytes) "
+					"does not match the slave's configured output_size (%d bytes)." % (
+					slaveDesc.slaveAddr,
+					len(fromSlaveData),
+					slaveDesc.outputSize))
+			slave.faultDeb.fault()
+			fromSlaveData = None
+		slave.fromSlaveData = fromSlaveData
 
 		return slaveDesc
 
-	def setSlaveOutData(self, slaveDesc, outData):
-		"""Set the out-data that will be sent the
+	def _setToSlaveData(self, slaveDesc, data):
+		"""Set the master-out-data that will be sent the
 		next time we are able to send something to that slave.
 		"""
+		if (data is not None and
+		    len(data) != slaveDesc.inputSize):
+			raise DpError("Slave %d: The setMasterOutData() data size (%d bytes) "
+				      "does not match the slave's configured input_size (%d bytes)." % (
+				      slaveDesc.slaveAddr,
+				      len(data),
+				      slaveDesc.inputSize))
 		slave = self.__slaveStates[slaveDesc.slaveAddr]
-		slave.outData = outData
+		slave.toSlaveData = data
 
-	def getSlaveInData(self, slaveDesc):
-		"""Get the latest received in-data.
+	def _getFromSlaveData(self, slaveDesc):
+		"""Get the latest received master-in-data.
 		Returns None, if there was no received data.
 		"""
 		slave = self.__slaveStates[slaveDesc.slaveAddr]
-		inData = slave.inData
-		slave.inData = None
-		return inData
+		fromSlaveData = slave.fromSlaveData
+		slave.fromSlaveData = None
+		return fromSlaveData
+
+	def _slaveIsConnecting(self, slaveDesc):
+		slave = self.__slaveStates[slaveDesc.slaveAddr]
+		return (not slave.dxCycleRunning and
+		        slave.getState() != DpSlaveState.STATE_INIT)
+
+	def _slaveIsConnected(self, slaveDesc):
+		slave = self.__slaveStates[slaveDesc.slaveAddr]
+		return slave.dxCycleRunning
 
 	def initialize(self):
 		"""Initialize the DPM."""
